from collections import OrderedDict

import graphene
from graphene_django.types import DjangoObjectType, DjangoObjectTypeOptions


class GeoJSONTypeOptions(DjangoObjectTypeOptions):
    geojson_field = None

    def __setattr__(self, name, value):
        if name == 'fields':
            geometry_field = value.pop(self.geojson_field)
            geometry_field.name = 'geometry'

            primary_key = self.model._meta.pk.name
            primary_key_field = value.pop(primary_key, None)

            Properties = type('Properties', (graphene.ObjectType,), value)

            fields = [
                ('type', graphene.Field(graphene.String)),
                (self.geojson_field, geometry_field),
                ('properties', graphene.Field(Properties)),
            ]

            if primary_key_field is not None:
                fields.insert(0, ('id', primary_key_field))

            value = OrderedDict(fields)

        super().__setattr__(name, value)


def feature_resolver(attname, default_value, root, info, **args):
    if attname == 'type':
        return 'Feature'
    elif info.field_name == 'geometry':
        return getattr(root, attname)
    return root


class GeoJSONType(DjangoObjectType):

    class Meta:
        abstract = True

    @classmethod
    def __init_subclass_with_meta__(cls, name=None, geojson_field=None,
                                    default_resolver=None, **options):

        assert geojson_field is not None, (
            '`{}` should either include a `Meta.geojson_field`'
            ' attribute.'.format(name or cls.__name__)
        )

        _meta = GeoJSONTypeOptions(cls)
        _meta.geojson_field = geojson_field

        super().__init_subclass_with_meta__(
            default_resolver=feature_resolver,
            name=name, _meta=_meta, **options)



#!/usr/bin/env python

import os
import re

from setuptools import find_packages, setup


def get_long_description():
    for filename in ('README.rst',):
        with open(filename, 'r') as f:
            yield f.read()


def get_version(package):
    with open(os.path.join(package, '__init__.py')) as f:
        pattern = r'^__version__ = [\'"]([^\'"]*)[\'"]'
        return re.search(pattern, f.read(), re.MULTILINE).group(1)


setup(
    name='django-graphql-geojson',
    version=get_version('graphql_geojson'),
    license='MIT',
    description='GeoJSON support for Django GraphQL',
    long_description='\n\n'.join(get_long_description()),
    author='mongkok',
    author_email='domake.io@gmail.com',
    maintainer='mongkok',
    url='https://github.com/flavors/django-graphql-geojson/',
    packages=find_packages(exclude=['tests*']),
    install_requires=[
        'Django>=1.11',
        'graphene-django==2.0.0.geojson',
    ],
    dependency_links=[
        'git+https://github.com/mongkok/graphene-django@default-meta'
        '#egg=graphene-django-2.0.0.geojson',
    ],
    classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
    ],
    zip_safe=False,
    tests_require=[
        'Django>=1.11',
    ],
)



import json

from django.contrib.gis.db.models import GeometryField

import graphene
from graphene.types.generic import GenericScalar
from graphene_django.converter import convert_django_field


def geojson_resolver(attname, default_value, root, info, **args):
    if default_value is not None:
        root = root or default_value
    return json.loads(root.geojson)[attname]


class GeoJSON(graphene.ObjectType):
    type = graphene.String()
    coordinates = GenericScalar()

    class Meta:
        default_resolver = geojson_resolver


@convert_django_field.register(GeometryField)
def convert_field_to_geojson(field, registry=None):
    return graphene.Field(
        GeoJSON,
        description=field.help_text,
        required=not field.null)
